﻿--[[

(C) Copyright 2010 Ryknzu (Ryknzu@gmail.com)

]]
--[[

Chitchat

Chitchat is an addon that works as a in-game answering machine. Can choose when 
Chitchat will begin recording incoming whispers. While recording choose how Chitchat
will notify you of a whisper. All incoming whispers are saved locally and can be viewed at any
time through an easy to use interface. Have a fully customized experience with many easy to 
understand options.

Some features include:
* Automatically start recording while afk or dnd.
* Always record whispers regardless of afk or dnd status.
* Hide incoming whispers while recording and/or display a notification.
* Have your afk or dnd message automatically set to what you want.

Coming Soon:
* Automatically start recording during boss fights. 
* Automatically start recording while in Town.
* More options for submode 'Hide and Notify', Choosing where to display(Chat,Raid Warning,etc) and Text Color.
* Choosing color of whispers within the interface.
* When clicking on custom hyperlink opens interface to the right name and date.

I appreciate any feedback, please email me at ryknzu@gmail.com with your questions, concerns or suggestions.

]]

--------------------
-- Register with Ace
Chitchat = LibStub("AceAddon-3.0"):NewAddon("Chitchat", 
	"AceConsole-3.0", 
	"AceEvent-3.0",  
	"AceHook-3.0",
	"AceTimer-3.0")
	
-- Load Localization
local L = LibStub("AceLocale-3.0"):GetLocale("Chitchat")
-- Initialize minimap icon
local icon = LibStub("LibDBIcon-1.0")

-----------------
--| Constants |--
-----------------
CHITCHAT_VERSION = GetAddOnMetadata("Chitchat", "Version")
BINDING_HEADER_CHITCHAT = L["CHITCHAT"]
BINDING_NAME_CHITCHAT = L["TOGGLE"]
MODE_OFF = 1
MODE_ON = 2
MODE_AUTO = 3
SUBMODE_OFF = 1
SUBMODE_HIDE = 2
SUBMODE_NOTIFY = 3
HYPERLINK_REF = format("%s:%s", L["CHITCHAT"], L["VIEW"])
HYPERLINK = format("|c00FFFF00|H%s|h%s|h|r",HYPERLINK_REF, L["VIEW_MSG"])

---------------
-- Option Table
local Options = 
{
    name = L["CHITCHAT"],
    handler = Chitchat,
    type  = "group",
    args = 
	{
		addon_desc = 
        { -- Sets the Answering Machine mode description
			name = L["OPT_ADDON_DESC"],
			order = 1,
            type = "description",
			width = "full",
        },
		header_general = 
		{ -- General Options Header
			name = L["OPT_HEADER_GENERAL"],
			order = 2,
			type = "header",
		},
		minimap = 
		{ -- Toggles the Answering Machine minimap icon
			name = L["OPT_MINIMAP"],
			desc = L["OPT_MINIMAP_DESC"],
			order = 3,
			type = "toggle",
			width = "normal",
			get = "GetMinimap",
			set = "SetMinimap"
		},
		toggle = 
		{ -- Toggles the Answering Machine UI
			name = L["OPT_TOGGLE"],
			desc = L["OPT_TOGGLE_DESC"],
			order = 4,
			type = "execute",
			func = "ToggleChitchatFrame"
		},
		afk_desc = 
        { -- Sets the Answering Machine AFK description
			name = L["OPT_AFK"],
			order = 5,
            type = "description",
			width = "normal",
        },
        afk = 
        { -- Sets the Answering Machine AFK message
			name = "",
			desc = L["OPT_AFK_DESC"],
			order = 6,
            type = "input",
			width = "normal",
            get = "GetAFK",
            set = "SetAFK",
        },
		dnd_desc = 
        { -- Sets the Answering Machine DND description
            name = L["OPT_DND"],
			order = 7,
            type = "description",
			width = "normal",
        },
		dnd = 
        { -- Sets the Answering Machine DND message
            name = "",
			desc = L["OPT_DND_DESC"],
			order = 8,
            type = "input",
			width = "normal",
            get = "GetDND",
            set = "SetDND",
        },
		desc_clear = {
			name = L["OPT_CLEAR_DESC"],
			order = 9,
			type = "description",
			width = "normal",
		},
		clear = {
			name = L["OPT_CLEAR"],
			name = L["OPT_CLEAR_DESC"],
			order = 10,
			type = "execute",
			func = "ClearLogs",
			confirm = true,
		},
		header_mode = 
		{
			name = L["OPT_HEADER_MODES"],
			order = 11,
			type = "header",
		},
		mode_desc = 
        { -- Sets the Answering Machine mode description
			name = L["OPT_MODE_DESC"],
			order = 12,
            type = "description",
			width = "normal",
        },
		mode = 
		{ -- Toggles the Answering Machine recording
			name = L["OPT_MODE"],
			desc = L["OPT_MODE_DESC"],
			order = 13,
            type = "select",
			width = "normal",
			values = {L["OPT_MODE_OFF"], L["OPT_MODE_ON"], L["OPT_MODE_AFK"]},
            get = "GetMode",
            set = "SetMode",
			style = "dropdown",
		},
		submode_desc = 
        { -- Sets the Answering Machine mode description
			name = L["OPT_SUBMODE_DESC"],
			order = 14,
            type = "description",
			width = "normal",
        },
		submode = 
		{ -- Toggles the Answering Machine recording
			name = L["OPT_SUBMODE"],
			desc = L["OPT_SUBMODE_DESC"],
			order = 15,
            type = "select",
			width = "normal",
			values = {L["OPT_SUBMODE_OFF"], L["OPT_SUBMODE_HIDE"], L["OPT_SUBMODE_NOTIFY"]},
            get = "GetSubmode",
            set = "SetSubmode",
			style = "dropdown",
		},
		options = 
		{ -- Toggles the Answering Machine option panel
			name = L["OPT_OPTIONS"],
			desc = L["OPT_OPTIONS_DESC"],
			order = -1,
			guiHidden = true,
			type = "execute",
			func = "ShowOptions",
		},
    },
}

-------------------
-- Profile Defaults
local Defaults = 
{
    profile = 
    {
		names = {},
		minimap = 
		{
			hide = false
		},
        afk = L["AFK_MESSAGE_DEFAULT"],
        dnd = L["DND_MESSAGE_DEFAULT"],
		mode = MODE_AUTO,
		submode = SUBMODE_OFF,
		unread = 0,
    },
}

-----------------------------
--| INIT, ENABLE, DISABLE |--
-----------------------------
---------------
-- OnInitialize
-- Called when the Addon is Loaded
function Chitchat:OnInitialize()
	-- Loads the database
    self.db = LibStub("AceDB-3.0"):New("ChitchatDB",Defaults, "Default", "factionrealm")
	
	self:InitializeChitchatFrame()
	
    -- Registers the Option Table
    LibStub("AceConfig-3.0"):RegisterOptionsTable("Chitchat", Options)
    -- Adds options to 'Addons' Tab
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Chitchat",L["OPT_ADDON_TITLE"])
	
    -- Slash Commands
    self:RegisterChatCommand("chitchat", "ChatCommand")
	
	-- Setup minimap icon
	self.dataObject = LibStub("LibDataBroker-1.1"):NewDataObject("AnsweringMachineIcon", { 
		type = "data source",
		text = "General",
		icon = "Interface\\Icons\\INV_Misc_EngGizmos_13.blp",
		OnClick = function (frame, button)
			if button == "RightButton" then
				-- Show Options Window
				self:ShowOptions()
			else
				-- Show Interface Window
				self:ToggleChitchatFrame()
			end
		end,
		OnTooltipShow = function (tooltip)
			self:OnTooltipShow(tooltip)
		end
	})
	icon:Register("AnsweringMachineIcon", self.dataObject, self.db.profile.minimap)
	
	-- Spam Control
	lastWhisper = ""
	-- is currently recording
	isRecording = false
	hyperNameIndex = -1
	hyperDateIndex = -1
	isViewingOptions = false
end

-----------
-- OnEnable
-- Called when the addon is Enabled
function Chitchat:OnEnable()
	-- Checks player AFK or DND status upon Enabling
	if(Chitchat.db.profile.mode == MODE_ON) then
		StartRecording()
	end
	if(UnitIsAFK("player") or UnitIsDND("player")) then
		if(Chitchat.db.profile.mode == MODE_AUTO) then
			StartRecording()
		end
	end
    -- Register events
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", ChatSystemMessage)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", ChatWhisperMessage)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_AFK", ChatStatusMessage)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_DND", ChatStatusMessage)
	Chitchat:RegisterEvent("PLAYER_FLAGS_CHANGED")
	self:RawHook(nil, "SetItemRef", true)
end

------------
-- OnDisable
-- Called when the addon is Disabled
function Chitchat:OnDisable()
	StopRecording()
	-- Unregister events
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", ChatSystemMessage)
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", ChatWhisperMessage)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_AFK", ChatStatusMessage)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_DND", ChatStatusMessage)
end


--------------------------
-----| EVENT HANDLERS |---
--------------------------

-----------------------
-- PLAYER_ENTERING_WORLD
-- Called whenever the player sees a loading screen
function Chitchat:PLAYER_ENTERING_WORLD (message, unit)
	MissedWhisper()
end

-------------
-- SetItemRef
-- Sets the item l
function Chitchat:SetItemRef(link, text, button)
	if (link == HYPERLINK_REF) then 
		if(button == "LeftButton") then
			--Chitchat:SendMessage("CHITCHAT_MSG_HYPERLINK_OPEN", hyperNameIndex, hyperDateIndex)
			Chitchat:ToggleChitchatFrame()
		elseif(button == "RightButton") then
			InterfaceOptionsFrame_OpenToCategory(Chitchat.optionsFrame)
		end
		return nil
	end
	return self.hooks.SetItemRef(link, text, button)
end

----------------
-- OnTooltipShow
-- Tooltip when over minimap icon
function Chitchat:OnTooltipShow (tooltip)
	tooltip:AddLine(L["CHITCHAT"])
	if(isRecording) then
		tooltip:AddLine(L["RECORDING_ON"])
	else
		tooltip:AddLine(L["RECORDING_OFF"])
	end
	tooltip:AddLine(L["INSTRUCT_LEFTCLICK"])
	tooltip:AddLine(L["INSTRUCT_RIGHTCLICK"])
end

-----------------------
-- PLAYER_FLAGS_CHANGED
-- Sets the afk or dnd status messages
function Chitchat:PLAYER_FLAGS_CHANGED (message, unit)
	if (unit == "player") then
		if(UnitIsAFK("player")) then
			SendChatMessage(Chitchat.db.profile.afk , "AFK");
		elseif(UnitIsDND("player")) then
			SendChatMessage(Chitchat.db.profile.dnd , "DND");
		end
	end
end

-------------------
-- CHAT_MSG_WHISPER
-- Handles incoming whisper
 function ChatWhisperMessage(self, event, arg1, arg2, ...)
    if(isRecording) then
		local cTime = date("%X")
		local cDate = date("%x")
		local hName = format("|Hplayer:%s|h[%s]|h", arg2, arg2)
		local timeMsg = format("[%s]%s: %s",cTime, hName, arg1)
		if(lastWhisper ~= timeMsg) then
			AddWhisper(arg2,cDate,timeMsg)
			if(Chitchat.db.profile.submode == SUBMODE_NOTIFY) then
				DEFAULT_CHAT_FRAME:AddMessage(L["NOTIFY_MSG_FIRST"] .. hName .. L["NOTIFY_MSG_LAST"] .. " " .. HYPERLINK, 1.0, 0.5, 1.0);
			end
			lastWhisper = timeMsg
		end
		if(Chitchat.db.profile.submode == SUBMODE_HIDE or Chitchat.db.profile.submode == SUBMODE_NOTIFY) then
			return true, arg1, arg2, ...
		end
	end
	return false, arg1, arg2, ...
end 

-------------------------------
-- CHAT_MSG_AFK or CHAT_MSG_DND
-- Handles incoming afk or dnd auto-repsonse
 function ChatStatusMessage(self, event, arg1, arg2, ...)
    if(isRecording) then
		if(Chitchat.db.profile.submode == SUBMODE_HIDE or Chitchat.db.profile.submode == SUBMODE_NOTIFY) then
			return true
		else
			return false, arg1, arg2, ...
		end
    end
	return false, arg1, arg2, ...
end 

------------------
-- CHAT_MSG_SYSTEM
-- Start/Stop recording based on afk/dnd status changes
function ChatSystemMessage(self, event, arg1, arg2, ...)
	-- Unit is AFK
	if (string.find(arg1, string.sub(MARKED_AFK_MESSAGE, 1, string.len(MARKED_AFK_MESSAGE) - 4), 1))  then
		if(Chitchat.db.profile.mode == MODE_AUTO) then
			-- Couldn't change afk message here, would cause WoW to close
			-- SendChatMessage(Chitchat.db.profile.afk , "AFK");
			StartRecording()
		end
	-- Unit is DND
	elseif (string.find(arg1, string.sub(MARKED_DND, 1, string.len(MARKED_DND) - 3), 1)) then
		if(Chitchat.db.profile.mode == MODE_AUTO) then
			-- Couldn't change afk message here, would cause WoW to close
			-- SendChatMessage(Chitchat.db.profile.dnd , "DND");
			StartRecording()
		end
	end
	-- Unit is no longer AFK or DND
	if (arg1 == CLEARED_AFK or arg1 == CLEARED_DND) then
		if(Chitchat.db.profile.mode == MODE_ON) then
			StartRecording()
		else
			StopRecording()
		end
	end
	return false, arg1, arg2, ...
end


--------------------------
---------| API |----------
--------------------------
--------------
-- IndexOfName
-- Find index of name
function IndexOfName(name)
	hyperNameIndex = 0
	for _, playerName in ipairs(Chitchat.db.profile.names) do
		hyperNameIndex = hyperNameIndex + 1
		if(playerName.playerName == name) then
		
		end
	end
end

--------------
-- IndexOfDate
-- Find index of date
function IndexOfDate(name)
	index = 0
	for _, playerName in ipairs(Chitchat.db.profile.names) do
		if(playerName.playerName == name) then
			for _, dated  in ipairs(playerName.dates) do 
				index = index + 1
				if(dated.theDate == today) then
					return index
				end
			end
		end
	end
	return -1
end

-----------------
-- StartRecording
-- Start recording whispers
function StartRecording()
	-- Check to make sure it is not running
	if(isRecording == false) then
		isRecording = true
	end
end

----------------
-- StopRecording
-- Stop Recording whispers
function StopRecording()
	-- Check to make sure it is running
	if(isRecording) then
		isRecording = false
		-- Will only display missed whispers while setting is Automatic Record
		if(Chitchat.db.profile.mode == MODE_AUTO) then
			MissedWhisper()
		end
	end
end

----------------
-- MissedWhisper
-- Display missed whispers to the chat screen if there is any. 
function MissedWhisper()
	if (Chitchat.db.profile.unread == 1) then
		DEFAULT_CHAT_FRAME:AddMessage(L["MISSED_MSG_FIRST"] .. Chitchat.db.profile.unread .. L["MISSED_MSG_LAST_SINGULAR"] .. HYPERLINK, 1.0, 0.5, 1.0);
	elseif (Chitchat.db.profile.unread > 1) then
		DEFAULT_CHAT_FRAME:AddMessage(L["MISSED_MSG_FIRST"] .. Chitchat.db.profile.unread .. L["MISSED_MSG_LAST_PLURAL"] .. HYPERLINK, 1.0, 0.5, 1.0);
	end
end

-------------
-- AddWhisper
-- Save the whisper
function AddWhisper(author, today, whisper)
	-- NOTE: There must be a cleaner more effient way to do this
	local foundAuthor = false
	local unreadIncrease  = 1
	if(Chitchat.db.profile.mod == MODE_ON) then
		Chitchat.db.profile.unread = 0
		unreadIncrease = 0
	end
	Chitchat.db.profile.unread = Chitchat.db.profile.unread + unreadIncrease
	-- Add whisper
	for _, playerName in ipairs(Chitchat.db.profile.names) do
		if(playerName.playerName == author) then
			playerName.unread = "|cFFFF7FFF"
			foundAuthor = true
			for _, dated  in ipairs(playerName.dates) do 
				if(dated.theDate == today) then
					dated.unread = "|cFFFF7FFF"
					if(dated.unreadCount ~= nil) then
						dated.unreadCount = dated.unreadCount + unreadIncrease
					else
						dated.unreadCount = 1
					end
					if(dated.unreadWhispers ~= nil) then
						local newWhispers = format("%s\n%s", dated.unreadWhispers, whisper)
						dated.unreadWhispers = newWhispers
					else
						dated.unreadWhispers = whisper
					end
					Chitchat:SendMessage("CHITCHAT_MSG_WHISPERS_UPDATED")
					return
				end
			end
		end
	end
	-- Add new date column
	if(foundAuthor) then
		for _, playerName in ipairs(Chitchat.db.profile.names) do
			if(playerName.playerName == author) then
				local newDates = { unread = "|cFFFF7FFF", theDate = today, whispers = "", unreadWhispers = whisper, unreadCount = unreadIncrease }
				tinsert(playerName.dates, 1, newDates)
				Chitchat:SendMessage("CHITCHAT_MSG_WHISPERS_UPDATED")
				return
			end
		end 
	-- Add new author and date
	else
		local newAuthor = {playerName = author, unread = "|cFFFF7FFF", dates = {{ unread = "|cFFFF7FFF", theDate = today, whispers = "", unreadWhispers = whisper, unreadCount = unreadIncrease }}}
		tinsert(Chitchat.db.profile.names, 1, newAuthor)
		Chitchat:SendMessage("CHITCHAT_MSG_WHISPERS_UPDATED")
		return
	end
	
end


--------------------------
--------| OPTIONS |-------
--------------------------
---------------
-- ChatCommand
-- Handles chat command input
function Chitchat:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(Chitchat.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(Chitchat, "chitchat", "Chitchat", input)
    end
end

---------------------------------
-- ToggleChitchatFrame
-- Shows or hides the Answering Machine Frame
function Chitchat:ToggleChitchatFrame()
	if ChitchatFrame:IsShown() then
		HideUIPanel(ChitchatFrame)
	else
		ShowUIPanel(ChitchatFrame)
	end
end

------------
-- ClearLogs
-- Clears all entries
function Chitchat:ClearLogs()
	Chitchat.db.profile.unread = 0
	wipe(Chitchat.db.profile.names)
	Chitchat:SendMessage("CHITCHAT_MSG_LOGS_CLEARED")
end

--------------
-- ShowOptions
-- Shows Options Panel
function Chitchat:ShowOptions(info)
	InterfaceOptionsFrame_OpenToCategory(Chitchat.optionsFrame)
end

------------------
-- Get/Set Minimap
-- Shows or hides the Minimap icon
function Chitchat:GetMinimap(info)
    return not self.db.profile.minimap.hide
end
function Chitchat:SetMinimap(info, value)
    self.db.profile.minimap.hide = not value
	if (self.db.profile.minimap.hide) then
		icon:Hide("AnsweringMachineIcon")
	else
		icon:Show("AnsweringMachineIcon")
	end
end

--------------
-- Get/Set AFK
-- Sets and Returns the AFK message
function Chitchat:GetAFK(info)
    return self.db.profile.afk
end
function Chitchat:SetAFK(info, newValue)
    self.db.profile.afk = newValue
end

--------------
-- Get/Set DND
-- Sets and Returns the DND message
function Chitchat:GetDND(info)
    return self.db.profile.dnd
end
function Chitchat:SetDND(info, newValue)
    self.db.profile.dnd = newValue
end

---------------
-- Get/Set Mode
-- Sets and Returns the mode
function Chitchat:GetMode(info)
	return self.db.profile.mode
end
function Chitchat:SetMode(info, value)
    self.db.profile.mode = value
	
	if(value == MODE_OFF) then
		StopRecording()
	elseif(value == MODE_AUTO) then
		if(UnitIsAFK("player") or UnitIsDND("player")) then
			StartRecording()
		else
			StopRecording()
		end
	elseif(value == MODE_ON) then
		StartRecording()
	end
end

------------------
-- Get/Set Submode
-- Sets and Returns the submode
function Chitchat:GetSubmode(info)
	return self.db.profile.submode
end
function Chitchat:SetSubmode(info, value)
    self.db.profile.submode = value
end