--[[

(C) Copyright 2010 Ryknzu (Ryknzu@gmail.com)

]]
-- Load Localization
local L = LibStub("AceLocale-3.0"):GetLocale("Chitchat")
-------------------------------------------------------------------

-----------------------------
--| INIT, ENABLE, DISABLE |--
-----------------------------
-------------
-- Initialize
-- Called when the addon is loaded
function Chitchat:InitializeChitchatFrame ()
	-- Update
	self:UpdateNames()
	self:RegisterMessage("CHITCHAT_MSG_WHISPERS_UPDATED")
	self:RegisterMessage("CHITCHAT_MSG_LOGS_CLEARED")
	self:RegisterMessage("CHITCHAT_MSG_HYPERLINK_OPEN")
end

---------
-- Enable
-- Called when the addon frame is enabled
function Chitchat:EnableChitchatFrame ()
	self:RegisterEvent("UNIT_PORTRAIT_UPDATE")
end

----------
-- Disable
-- Called when the addon frame is disabled
function Chitchat:DisableChitchatFrame ()
	self:UnregisterEvent("UNIT_PORTRAIT_UPDATE")	
end


-----------------------------
-----| EVENT HANDLERS |------
-----------------------------
---------
-- OnLoad
-- Called when the addon frame is loaded
function Chitchat:OnLoadChitchatFrame (frame)
	ChitchatFrame:SetAttribute("UIPanelLayout-defined", true)
	ChitchatFrame:SetAttribute("UIPanelLayout-enabled", true)
	ChitchatFrame:SetAttribute("UIPanelLayout-area", "left")
	ChitchatFrame:SetAttribute("UIPanelLayout-pushable", 5)
	ChitchatFrame:SetAttribute("UIPanelLayout-whileDead", true)
	ChitchatFrameTitleText:SetText(L["CHITCHAT"])
	ChitchatDeleteNameButton:SetText(L["DELETE_NAME"])
	ChitchatDeleteDateButton:SetText(L["DELETE_DATE"])
end

---------
-- OnShow
-- Called when the addon frame is shown
function Chitchat:OnShowChitchatFrame (frame)
	PlaySound("igCharacterInfoOpen")
	SetPortraitTexture(ChitchatFramePortrait, "player")
end

---------
-- OnHide
-- Called when the addon frame is hidden
function Chitchat:OnHideChitchatFrame (frame)
	PlaySound("igCharacterInfoClose")
end

---------
-- OnHide
-- Called when the addon frame is hidden
function Chitchat:OnHideChitchatFrame (frame)
	PlaySound("igCharacterInfoClose")
end

--------------------
-- OnClickDeleteName
-- Handles the clicking of the delete button
function Chitchat:OnClickDeleteName (frame)
	if(Chitchat.selectedName ~= nil) then
		tremove(Chitchat.db.profile.names,Chitchat.selectedNameIndex - 1)	
		Chitchat.db.profile.unread = 0
		Chitchat:UpdateNames()
		Chitchat:UpdateDates()
		Chitchat:UpdateWhispers()
	end
	PlaySound("igMainMenuOptionCheckBoxOn");
end

--------------------
-- OnClickDeleteDate
-- Handles the clicking of the delete button
function Chitchat:OnClickDeleteDate (frame)
	if(Chitchat.selectedDate ~= nil) then
		tremove(Chitchat.selectedName.dates,Chitchat.selectedDateIndex - 1)	
		Chitchat:UpdateDates()
		Chitchat:UpdateWhispers()
	end
	PlaySound("igMainMenuOptionCheckBoxOn");
end

---------------------------------
-- CHITCHAT_MSG_WHISPERS_UPDATED
-- Handles event when whisper is added or removed
function Chitchat:CHITCHAT_MSG_WHISPERS_UPDATED()
	Chitchat_InitializeNameDropDown()
	Chitchat_InitializeDateDropDown()
	Chitchat:UpdateWhispers()
end

-----------------------------
-- CHITCHAT_MSG_LOGS_CLEARED
-- Handles event when all whisper data is cleared
function Chitchat:CHITCHAT_MSG_LOGS_CLEARED()
	Chitchat:UpdateNames()
end

-----------------------------
-- CHITCHAT_MSG_HYPERLINK_OPEN
-- Handles event when open by a hyperlink
function Chitchat:CHITCHAT_MSG_HYPERLINK_OPEN(hyperName, hyperDate)

end

-----------------------
-- UNIT_PORTRAIT_UPDATE
-- Updates the frame portrait
function Chitchat:UNIT_PORTRAIT_UPDATE (message, unit)
	if unit == "player" then
		SetPortraitTexture(ChitchatFramePortrait, "player")
	end
end


----------------------------
--| SELECTION MANAGEMENT |--
----------------------------
local UIDropDownTable ={}
--------------
-- UpdateNames
-- Updates the Names drop down
function Chitchat:UpdateNames (limitted)
	self.selectedName = nil
	self.selectedNameIndex = 1
	ChitchatDeleteNameButton:Disable()
	UIDropDownMenu_Initialize(ChitchatNameDropDown, Chitchat_InitializeNameDropDown)
	UIDropDownMenu_SetSelectedID(ChitchatNameDropDown, self.selectedNameIndex)
	if not limitted then
		self:UpdateDates()
	end
end

-------------------------
-- InitializeNameDropDown
-- Initializes the Names drop down
function Chitchat_InitializeNameDropDown ()
	local info = UIDropDownMenu_CreateInfo()
	info.text = "Select Name"
	info.func = Chitchat_OnNameDropDownClick
	UIDropDownMenu_AddButton(info)
	
	for _, playerName in ipairs(Chitchat.db.profile.names) do 
		info = UIDropDownMenu_CreateInfo()
		info.text = playerName.playerName
		info.func = Chitchat_OnNameDropDownClick
		info.value = playerName
		-- Added: 3/19/10
		-- Color the player's name, Normally: Purple is unread, white is read.
		if(playerName.unread ~= nil) then
			info.colorCode = playerName.unread
		end
		UIDropDownMenu_AddButton(info)
	end
end

----------------------
-- OnNameDropDownClick
-- Handles the selection of an Name
function Chitchat_OnNameDropDownClick (button)
	UIDropDownMenu_SetSelectedID(ChitchatNameDropDown, button:GetID())
	Chitchat.selectedNameIndex = button:GetID()
	Chitchat.selectedName = button.value
	ChitchatDeleteNameButton:Enable()
	if type(Chitchat.selectedName) ~= "table" then
		Chitchat.selectedName = nil
		ChitchatDeleteNameButton:Disable()
	else
		-- Added: 3/19/10
		-- The whisper is being read, sets the color to white.
		Chitchat.selectedName.unread = "|cFFFFFFFF"
	end
	Chitchat:UpdateDates()
end

--------------
-- UpdateDates
-- Updates the dates drop down
function Chitchat:UpdateDates (limitted)
	self.selectedDate = nil
	self.selectedDateIndex = 1
	ChitchatDeleteDateButton:Disable()
	UIDropDownMenu_Initialize(ChitchatDateDropDown, Chitchat_InitializeDateDropDown)
	UIDropDownMenu_SetSelectedID(ChitchatDateDropDown, self.selectedDateIndex)
	if not limitted then
		Chitchat:UpdateWhispers()
	end
end

-------------------------
-- InitializeDateDropDown
-- Initializes the dates drop down
function Chitchat_InitializeDateDropDown ()
	-- Add "All Dates"
	local info = UIDropDownMenu_CreateInfo()
	info.text = "Select Date"
	info.func = Chitchat_OnDateDropDownClick
	UIDropDownMenu_AddButton(info)
	-- Add dates encounters
	if Chitchat.selectedName then
		for _, dated in ipairs(Chitchat.selectedName.dates) do
			info = UIDropDownMenu_CreateInfo()
			info.text = dated.theDate
			info.func = Chitchat_OnDateDropDownClick
			info.value = dated
			-- Added: 3/19/10
			-- Color the date, Normally: Purple is unread, white is read.
			if(dated.unread ~= nil) then
				info.colorCode = dated.unread
			end
			UIDropDownMenu_AddButton(info)
		end
	end
end

----------------------
-- OnDateDropDownClick
-- Handles the selection of an date
function Chitchat_OnDateDropDownClick (button)
	UIDropDownMenu_SetSelectedID(ChitchatDateDropDown, button:GetID())
	Chitchat.selectedDateIndex = button:GetID()
	Chitchat.selectedDate = button.value
	ChitchatDeleteDateButton:Enable()
	if type(Chitchat.selectedDate) ~= "table" then
		Chitchat.selectedDate = nil
		ChitchatDeleteDateButton:Disable()
	else
		-- Added: 3/19/10
		-- The whisper is being read, sets the color to white.
		Chitchat.selectedDate.unread = "|cFFFFFFFF"
	end
	Chitchat:UpdateWhispers()
end

-----------------
-- UpdateWhispers
-- Update the list of whispers
function Chitchat:UpdateWhispers (limitted)
	ChitchatNote:SetTextColor(1.0, 1.0, 1.0)
	if(Chitchat.selectedDate ~= nil) then
		-- Added: 3/19/10
		-- Upgrade check
		if(Chitchat.db.profile.unread ~= nil and Chitchat.selectedDate.unreadCount ~= nil) then
			Chitchat.db.profile.unread = Chitchat.db.profile.unread - Chitchat.selectedDate.unreadCount
			Chitchat.selectedDate.unreadCount = 0
			if(Chitchat.db.profile.unread < 0) then
				Chitchat.db.profile.unread = 0
			end
		else
			Chitchat.db.profile.unread = 0
		end
		local whisper
		-- Added: 3/19/10
		-- Upgrade check
		if(Chitchat.selectedDate.whispers ~= nil and Chitchat.selectedDate.unreadWhispers ~= nil) then
			-- Combines the Read whispers with the unread whispers and sets the unread color to purple
			whisper = format("%s\n|cFFFF7FFF%s|r",Chitchat.selectedDate.whispers,Chitchat.selectedDate.unreadWhispers)
			if(Chitchat.selectedDate.unreadWhispers ~= "") then
				Chitchat.selectedDate.whispers = format("%s\n%s",Chitchat.selectedDate.whispers,Chitchat.selectedDate.unreadWhispers)
			else
				Chitchat.selectedDate.whispers = Chitchat.selectedDate.whispers
			end
			Chitchat.selectedDate.unreadWhispers = ""
			Chitchat.selectedDate.unread = "|cFFFFFFFF"
			Chitchat.selectedName.unread = "|cFFFFFFFF"
		else
			whisper = Chitchat.selectedDate.whispers
		end
		whisper = gsub(whisper, "\n\n", "\n")
		ChitchatNote:SetText(whisper)
	else
		ChitchatNote:SetText("")
	end
end